@@ -49,32 +49,7 @@ module GNOME2
       private
       def define_build_tasks
         namespace :build do
-          prepare_task_names = []
-          namespace :prepare do
-            prepare_task_names << "pkg_config"
-            task :pkg_config do
-              depended_packages = @package.windows.build_dependencies
-              use_packages = [@package.name] + depended_packages
-              pkg_config_path = use_packages.collect do |package|
-                "../#{package}/#{@package.windows.relative_binary_dir}/lib/pkgconfig"
-              end
-              ENV["PKG_CONFIG_PATH"] = pkg_config_path.collect do |path|
-                File.expand_path(path)
-              end.join(":")
-              ENV["PKG_CONFIG_LIBDIR"] = rcairo_pkgconfig_path
-            end
-
-            prepare_task_names << "pkg_config_for_build"
-            task :pkg_config_for_build do
-              # XXX: Is it needless?
-              # ENV["PKG_CONFIG_FOR_BUILD"] = "env - pkg-config"
-            end
-          end
-
-          full_prepare_task_names = prepare_task_names.collect do |name|
-            "windows:builder:build:prepare:#{name}"
-          end
-          task :prepare => full_prepare_task_names
+          task :prepare
 
           build_packages.each do |package|
             namespace package.name do
@@ -118,6 +93,7 @@ module GNOME2
           sh("tar", "xf", tar_full_path.to_s)
         end
 
+        env = build_env
         package_dir_path =
           package_tmp_dir + package.base_name + package.base_dir_in_package
         Dir.chdir(package_dir_path.to_s) do
@@ -131,36 +107,36 @@ module GNOME2
             common_make_args << cc_env(package)
           end
           add_gobject_introspection_make_args(package, common_make_args)
+          common_make_args.concat(package.windows.make_args)
           build_make_args = common_make_args.dup
           install_make_args = common_make_args.dup
           if package.windows.build_concurrently?
             make_n_jobs = ENV["MAKE_N_JOBS"]
             build_make_args << "-j#{make_n_jobs}" if make_n_jobs
           end
-          ENV["GREP_OPTIONS"] = "--text"
-          # ENV["GI_SCANNER_DEBUG"] = "save-temps"
-          if File.exist?("meson.build") and not File.exist?("Makefile.am")
+          # build_make_args << "--debug"
+          # build_make_args << "V=1"
+          # build_make_args << "VERBOSE=1"
+          # env["GI_SCANNER_DEBUG"] = "save-temps"
+          if File.exist?("meson.build")
             source_dir = Dir.pwd
             build_dir = "build"
             mkdir_p(build_dir)
             Dir.chdir(build_dir) do
-              meson(package, source_dir)
-              sh(make_args_to_env(build_make_args),
+              meson(env, package, source_dir)
+              sh(env.merge(make_args_to_env(build_make_args)),
                  "nice", "ninja") or exit(false)
-              sh(make_args_to_env(install_make_args),
+              sh(env.merge(make_args_to_env(install_make_args)),
                  "ninja", "install") or exit(false)
             end
           else
-            if File.exist?("Makefile.in")
-              configure(package)
+            if File.exist?("Makefile.am") or File.exist?("Makefile.in")
+              configure(env, package)
             else
-              cmake(package)
+              cmake(env, package)
             end
-            # build_make_args << "--debug"
-            # build_make_args << "V=1"
-            # build_make_args << "VERBOSE=1"
-            sh("nice", "make", *build_make_args) or exit(false)
-            sh("make", "install", *install_make_args) or exit(false)
+            sh(env, "nice", "make", *build_make_args) or exit(false)
+            sh(env, "make", "install", *install_make_args) or exit(false)
           end
 
           package_license_dir = license_dir + package.name
@@ -182,26 +158,61 @@ module GNOME2
         end
       end
 
-      def configure(package)
-        sh("./autogen.sh") if package.windows.need_autogen?
-        sh("autoreconf", "--install", "--force") if package.windows.need_autoreconf?
-        sh("./configure",
+      def build_env
+        path = "#{g_ir_scanner_bin_dir}:#{ENV["PATH"]}"
+        env = {
+          "GI_HOST_OS" => "nt",
+          "MSYSTEM" => "MINGW64",
+          "GREP_OPTIONS" => "--text",
+          "PATH" => path,
+          "PKG_CONFIG" => pkg_config,
+        }
+        depended_packages = @package.windows.build_dependencies
+        use_packages = [@package.name] + depended_packages
+        pkg_config_path = [rcairo_pkgconfig_path]
+        use_packages.each do |package|
+          path = "../#{package}/#{@package.windows.relative_binary_dir}/lib/pkgconfig"
+          pkg_config_path << path
+        end
+        env["PKG_CONFIG_PATH"] = pkg_config_path.collect do |path|
+          File.expand_path(path)
+        end.join(":")
+        env
+      end
+
+      def configure(env, package)
+        sh(env, "./autogen.sh") if package.windows.need_autogen?
+        if package.windows.need_autoreconf?
+          sh(env, "autoreconf", "--install", "--force")
+        end
+        sh(env, "env")
+        sh(env,
+           "./configure",
            cc_env(package),
            dlltool_env,
            "CPPFLAGS=#{cppflags(package)}",
            "LDFLAGS=#{ldflags(package)}",
            "--prefix=#{dist_dir}",
            "--host=#{@package.windows.build_host}",
+           # "--target=#{@package.windows.build_host}",
+           # "--build=x86_64-pc-linux-gnu",
            *package.windows.configure_args) or exit(false)
+        if package.windows.force_to_disable_deplibs_check?
+          sh(env,
+             "sed",
+             "-i",
+             "-e", "s/valid_a_lib=false/valid_a_lib=:/g",
+             "libtool")
+        end
       end
 
-      def meson(package, source_dir)
+      def meson(env, package, source_dir)
         cross_file = Tempfile.new("meson-cross-file")
         cross_file.puts(<<-CROSS_FILE)
 [host_machine]
 system = 'windows'
-cpu_family = 'x86_64'
-cpu = 'x86_64'
+cpu_family = '#{meson_cpu_family}'
+cpu = '#{meson_cpu}'
 endian = 'littel'
 
 [binaries]
@@ -210,24 +221,48 @@ cpp = '/usr/bin/#{cxx(package)}'
 ar = '/usr/bin/#{ar}'
 strip = '/usr/bin/#{strip}'
 dlltool = '/usr/bin/#{dlltool}'
-pkgconfig = '/usr/bin/pkg-config'
+pkgconfig = '/usr/bin/#{pkg_config}'
+windres = '/usr/bin/#{windres}'
+exe_wrapper = 'wine'
+
+[properties]
+root = '/usr/#{@package.windows.build_host}'
         CROSS_FILE
         cross_file.close
-        sh({
-             "PATH" => "#{g_ir_scanner_bin_dir}:#{ENV["PATH"]}",
-             "CPPFLAGS" => cppflags(package),
-             "LDFLAGS" => ldflags(package),
-           },
+        sh(env.merge({
+                       "CPPFLAGS" => cppflags(package),
+                       "LDFLAGS" => ldflags(package),
+                     }),
            "meson",
            source_dir,
            ".",
            "--prefix=#{dist_dir}",
+           "--libdir=lib",
            "--cross-file=#{cross_file.path}",
            *package.windows.meson_args) or exit(false)
       end
 
-      def cmake(package)
-        sh("cmake",
+      def meson_cpu_family
+        case @package.windows.build_architecture
+        when "x86"
+          "x86"
+        when "x86_64"
+          "x86_64"
+        end
+      end
+
+      def meson_cpu
+        case @package.windows.build_architecture
+        when "x86"
+          "i686"
+        when "x86_64"
+          "x86_64"
+        end
+      end
+
+      def cmake(env, package)
+        sh(env,
+           "cmake",
            ".",
            "-DCMAKE_INSTALL_PREFIX=#{dist_dir}",
            "-DCMAKE_SYSTEM_NAME=Windows",
@@ -270,8 +305,7 @@ pkgconfig = '/usr/bin/pkg-config'
       end
 
       def rcairo_dir
-        suffix = @package.windows.build_architecture_suffix
-        @package.project_root_dir.parent + "rcairo.#{suffix}"
+        @package.project_root_dir.parent + "rcairo"
       end
 
       def rcairo_pkgconfig_path
@@ -313,6 +347,14 @@ pkgconfig = '/usr/bin/pkg-config'
         "#{@package.windows.build_host}-dlltool"
       end
 
+      def pkg_config
+        "#{@package.windows.build_host}-pkg-config"
+      end
+
+      def windres
+        "#{@package.windows.build_host}-windres"
+      end
+
       def g_ir_scanner_bin_dir
         "#{@package.project_root_dir}/gobject-introspection/" +
           "#{@package.native.relative_binary_dir}/bin"
